class Balloon < Peta::Element
  load_manifest
  belongs_to :speech_balloon
  belongs_to :speech_balloon_template
  belongs_to :system_picture
  
  validates :speech_balloon_id, :numericality => {:allow_blank => true}
  validates :speech_balloon_template_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :speech_balloon_template_module_name, :presence => true, :length => {:maximum => 50}
  validates :system_picture_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :r, :presence => true, :numericality => true
#  validates :caption, :presence => true
  validates :speech_balloon_template_settings, :boost => {:boost_name => :speech_balloon_template}

  scope :with_panel, -> do
    includes(speech_balloon: :panel)
  end
  
  scope :with_speech_balloon_template, -> do
    includes(:speech_balloon_template)
  end
  
  scope :find_index, -> do
    with_panel.where(Panel.arel_table[:publish].gt 0).references(:panel)
  end
  
  scope :find_private, -> (operators) do 
    with_panel.where(Panel.arel_table[:author_id].eq operators.author.id).references(:panel)
  end
  
  scope :find_by_panel, -> (panel_id) do 
    find_index.where(panel_id: panel_id).references(:panel)
  end
  
  scope :find_by_speech_balloon, -> (speech_balloon_id) do 
    find_index.where(speech_balloon_id: speech_balloon_id).references(:panel)
  end
  
  scope :find_by_speech_balloon_template, -> (speech_balloon_template_id) do 
    find_index.where(speech_balloon_template_id: speech_balloon_template_id).references(:panel)
  end
  
  scope :find_by_system_picture, -> (system_picture_id) do 
    find_index.where(system_picture_id: system_picture_id).references(:panel)
  end
  
  scope :find_by_author, -> (author_id) do 
    find_index.where(Panel.arel_table[:author_id].eq author_id).references(:panel)
  end
  
  def y
    self.attributes['y']
  end
  
  def url
    '/system_pictures/' + self.system_picture.filename
  end
  
  def supply_default
    self.x = 0
    self.y = 0
    self.width = 100
    self.height = 100
    self.r = 0
self.system_picture_id = 1
  end
  
  def symbol_option
    self.speech_balloon.speech_balloon_template.symbol_option
  end
  
  def self.public_list_where list
    'panels.publish > 0'
  end
  
  def self.by_author_list_includes
    {
      :speech_balloon => {
        :panel => {
          :author => {}
        }
      }
    }
  end
  
  def self.show_opt
    {:include => {:speech_balloon => {:panel => {:author => {}}, :speech => {}, :speech_balloon_template => {} }}}
  end
  
  def style
    {
      'width' => self.width.to_s + 'px','height' => self.height.to_s + 'px',
      'top' => self.y.to_s + 'px','left' => self.x.to_s + 'px'
    }
  end
  
  def scenario
  end
  
  def plain_scenario
  end
  
end
