ManifestBase.require_modules "manifest/model/", 
  %w|attribute association|

module Manifest
  class Model < ManifestBase::Base
    include ModelModule
    
    attr :attributes, :associations, :table_name
    
    def set_default
      super
      @json['table_name'] ||= ::Manifest.manifest.pluralize(@name)
      @json['attributes'] ||= {}
      @json['associations'] ||= {}
      @json['attributes']['id'] = {
        'type' => 'number',
        'primary_key' => 1,
        'rules' => {
          'number' => true
        }
      }
      @json['attributes']['created_at'] = {
        'type' => 'datetime',
      }
      @json['attributes']['updated_at'] = {
        'type' => 'datetime',
      }
    end
    
    def init
      super
      @table_name = @json['table_name']
      @attributes = ManifestBase.load_name_values self, @json, 'attributes', Attribute
      @associations = ManifestBase.load_value self, @json, 'associations', Association
    end
    
    def item_name
      @name
    end
    
    def classify
      ::Manifest.item_name_to_model @name
    end
    
    def valid_encode_columns
      r = []
      @attributes.each {|attribute_name, attribute|
        next unless attribute.type == 'text'
        r << attribute_name
      }
      r
    end
    
    def owner_type
      return :author if @attributes['author_id']
      return :artist if @attributes['artist_id']
      false
    end
    
    def content_model?
      return true if self.owner_type
      false
    end
    
    def child_model_manifests
      r = []
      ::Manifest.manifest.items.each {|peta_name, peta_manifest|
        next unless peta_manifest.element?
        next unless peta_manifest.parent_item_name == @name
        r << ::Manifest.manifest.models[peta_name]
      }
      r
    end
    
    def child_models
      self.child_model_manifests.map {|child_model_manifest|
        child_model_manifest.classify
      }
    end
    
    def child_element_names
      self.child_models.map {|child_model|
        self.associations.child_element_name(child_model)
      }
    end
    
    def child_element_name item_name
      self.associations.child_element_name(child_model_manifest)
    end
    
  end
end

